# Set working directory:
setwd("~/")


# Make table of settings:

method <- c("ordforest", "naiveordforest", "multiclassforest", "ordproblog")
dat <- c("mammography_dataset.Rda", "supportstudy_dataset.Rda", 
         "vlbw_dataset.Rda", "whinequality_dataset.Rda", "nhanes_dataset.Rda")
cvind <- 1:10

scenariogrid <- expand.grid(cvind=cvind, method=method, dat=dat, stringsAsFactors = FALSE)
scenariogrid <- scenariogrid[,ncol(scenariogrid):1]

set.seed(1234)
seeds <- sample(1000:10000000, size=length(cvind))

scenariogrid$seed <- rep(seeds, times=length(method)*length(dat))



# Randomly permute rows of the table containing the settings.
# This is performed to ensure a comparable computational burdens for
# the jobs to be performed in parallel:

set.seed(1234)
reorderind <- sample(1:nrow(scenariogrid))
scenariogrid <- scenariogrid[reorderind,]



# Source the functions that are used in performing the calculations 
# on the cluster:

source("./HornungOrdinalForests/Functions/FunctionsRealDataAnalysis.R")


# Export all objects to the CPUs:

sfExportAll()


# Evaluate the settings:

Results <- sfLapply(1:nrow(scenariogrid), function(z) try({evaluatesetting(z)}))


# Stop the cluster:

sfStop()


# Save the results:

save(Results, file="./HornungOrdinalForests/Results/RealDataAnalysis_Results.Rda")



# Clear the workspace:

rm(list=ls());gc()
