# In case the folder "HornungOrdinalForests" is not contained in the
# current R working directory, use the following line and replace
# *directory that contains the folder 'HornungOrdinalForests'* by
# the path to the directory that contains the folder
# "HornungOrdinalForests":

# setwd("*directory that contains the folder 'HornungOrdinalForests'*")


# Dataset 'vlbw':
#################

# Source: http://biostat.mc.vanderbilt.edu/wiki/Main/DataSets
# (last accessed: 15th October 2017)

# Load raw dataset:
load("./HornungOrdinalForests/RealDataApplications/data/vlbw.sav")

# include only prenatal variables
vlbw <- vlbw[,names(vlbw) %in% c("apg1", "twn", "inout", "sex", "race", "delivery", "meth", "lol", "magsulf", "bwt", "toc")]

# convert variables to the correct data format
vlbw$twn     <- as.factor(vlbw$twn)
vlbw$magsulf <- as.factor(vlbw$magsulf)
vlbw$meth    <- as.factor(vlbw$meth)
vlbw$toc     <- as.factor(vlbw$toc)

# exclude observations with missing information
vlbw <- vlbw[complete.cases(vlbw),]

# exclude observations with too sparse levels
vlbw <- vlbw[vlbw$apg1 != 0,]                 # only 2 observations with apgar = 0
vlbw <- vlbw[vlbw$race != "oriental",]        # only 1 observation with race = oriental
vlbw <- vlbw[vlbw$race != "native American",] # only 4 observations with race = native American
vlbw$race <- factor(vlbw$race)


# specify response as nominal variable
vlbw$apg1 <- factor(vlbw$apg1, ordered = FALSE)

table(vlbw$apg1)

dataset <- vlbw
dataset$y <- dataset$apg1
dataset$apg1 <- NULL

# Save pre-processed data to be used in the real data analysis:
save(dataset, file="./HornungOrdinalForests/RealDataApplications/data/vlbw_dataset.Rda")

rm(list=ls());gc()




# Dataset 'winequality':
########################

# Source: https://github.com/zygmuntz/wine-quality/blob/master/winequality/winequality-white.csv 
# (last accessed: 15th October 2017)

# Load dataset:
wine <- read.csv("./HornungOrdinalForests/RealDataApplications/data/winequality-white.csv", header = TRUE, sep = ";")

# exclude 5 observations with quality = 9 (too few observations in that class)
wine <- wine[wine$quality < 9,]

# specify response as nominal variable
wine$quality <- factor(wine$quality, ordered = FALSE)


dataset <- wine
dataset$y <- dataset$quality
dataset$quality <- NULL

# Save pre-processed data to be used in the real data analysis:
save(dataset, file="./HornungOrdinalForests/RealDataApplications/data/whinequality_dataset.Rda")

rm(list=ls());gc()





# Dataset 'nhanes':
###################

# Source: Supplementary Materials of "S. Janitza, H. Binder, A.-L. Boulesteix, 2015. 
# Pitfalls of Hypothesis Tests and Model Selection on Bootstrap Samples: 
# Causes and Consequences in Biometrical Applications. Biometrical Journal".

# Load dataset:
load("./HornungOrdinalForests/RealDataApplications/data/NHANES_data.Rda")


dim(data)

#-------------------------------------------------------------------------------------------------------------------------

# The considered data dataset contains the following variables:
#
# > names(data)
# [1] "sex"               "race"              "country_of_birth"  "education"         "marital_status"    "waistcircum"      
# [7] "Cholesterol"       "WBCcount"          "HealthStatus"      "AcuteIllness"      "depression"        "ToothCond"        
# [13] "sleepTrouble"      "wakeUp"            "cig"               "diabetes"          "asthma"            "heartFailure"     
# [19] "stroke"            "chronicBronchitis" "alcohol"           "heavyDrinker"      "medicalPlaceToGo"  "BPsys"            
# [25] "BPdias"            "age"               "BMI" 

# see Table A1 in Janitza, Binder and Boulesteix (2014) 'Pitfalls of Hypothesis Tests and Model Selection on Bootstrap Samples'
# (Technical Report 163, University of Munich) for a description on these variables. Note: variables CRP and income were not included

#-------------------------------------------------------------------------------------------------------------------------

varnames <- c("sex", "race", "country_of_birth" ,"education", "marital_status", 
 "waistcircum", "Cholesterol", "WBCcount", "HealthStatus", "AcuteIllness", 
 "depression", "ToothCond", "sleepTrouble", "wakeUp","cig", "diabetes", 
 "asthma", "heartFailure", "stroke", "chronicBronchitis", "alcohol", 
 "heavyDrinker", "medicalPlaceToGo", "BPsys", "BPdias", "age", "BMI")

all(varnames %in% names(data))
varnames[!(varnames %in% names(data))]
names(data)[!(names(data) %in% varnames)]

# --> "100cig" in "NHANES_data" seems to refer to "cig".

names(data)[names(data)=="100cig"] <- "cig"

all(varnames %in% names(data))

# --> OK.

data <- data[,names(data) %in% varnames]


# specify response as nominal variable
data$HealthStatus <- factor(data$HealthStatus, ordered = FALSE)


dataset <- data
dataset$y <- dataset$HealthStatus
dataset$HealthStatus <- NULL

# Save pre-processed data to be used in the real data analysis:
save(dataset, file="./HornungOrdinalForests/RealDataApplications/data/nhanes_dataset.Rda")

rm(list=ls());gc()





# Dataset 'supportstudy':
#########################

# Source: http://biostat.mc.vanderbilt.edu/wiki/Main/DataSets
# (last accessed: 15th October 2017)


# Load dataset:
load("./HornungOrdinalForests/RealDataApplications/data/support.sav")

# use only the following variables
support <- support[, names(support) %in% c("sfdm2", "age", "sex", "dzgroup", "dzclass", "num.co", "race", "charges", "scoma",
                                           "avtisst", "meanbp", "wblc", "hrt", "resp", "temp", "crea", "sod")]

# exclude variables with more than 10% missing values
support <- support[, c(names(apply(support[, names(support) != "sfdm2"], 2, function(z) sum(is.na(z)) <= floor(0.1 * (nrow(support)-1)))), "sfdm2")]


# complete case analyses
support <- support[complete.cases(support),]


# specify response as nominal variable
support$sfdm2 <- factor(support$sfdm2, ordered = FALSE)



table(support$dzgroup)
table(support$dzclass)

table(support$dzgroup, support$dzclass)

# --> "dzclass" is merely a coarsened version of "dzgroup".
# --> Exclude "dzclass":

support$dzclass <- NULL



dataset <- support
dataset$y <- support$sfdm2
dataset$sfdm2 <- NULL


# Save pre-processed data to be used in the real data analysis:
save(dataset, file="./HornungOrdinalForests/RealDataApplications/data/supportstudy_dataset.Rda")

rm(list=ls());gc()





# Dataset 'mammography':
#########################

# Source: R package 'TH.data'

# obtain Mammography Experience Data
library("TH.data")
data(mammoexp)

# specify response as nominal variable
mammoexp$ME <- factor(mammoexp$ME, ordered = FALSE)


dataset <- mammoexp
dataset$y <- mammoexp$ME
dataset$ME <- NULL

# Save pre-processed data to be used in the real data analysis:
save(dataset, file="./HornungOrdinalForests/RealDataApplications/data/mammography_dataset.Rda")

rm(list=ls());gc()
