# Set working directory:

setwd("~/")


# Make table of settings:

usecorrelations_highdim_numberobs <- c("correlations_nonhighdim_200", "nocorrelations_nonhighdim_200", 
                                       "correlations_nonhighdim_400", "nocorrelations_nonhighdim_400", 
                                       "correlations_highdim")
equalclasswidths <- c(TRUE, FALSE)
numberclass <- c(3,6,9)

method <- c("ordforest", "naiveordforest", "multiclassforest")

simit <- 1:100


scenariogrid <- expand.grid(method=method, simit=simit, equalclasswidths=equalclasswidths,  
                            numberclass=numberclass, usecorrelations_highdim_numberobs=usecorrelations_highdim_numberobs, stringsAsFactors = FALSE)
scenariogrid <- scenariogrid[,ncol(scenariogrid):1]



set.seed(1234)
seeds <- sample(1000:10000000, size=length(simit)*length(equalclasswidths)*length(numberclass)*length(usecorrelations_highdim_numberobs))

scenariogrid$seed <- rep(seeds, each=length(method))




# Randomly permute rows of the table containing the settings.
# This is performed to ensure a comparable computational burdens for
# the jobs to be performed in parallel:

set.seed(1234)
reorderind <- sample(1:nrow(scenariogrid))
scenariogrid <- scenariogrid[reorderind,]




# Source the functions that are used in performing the calculations 
# on the cluster:

source("./HornungOrdinalForests/Functions/FunctionsSimulation.R")




# Generate 50,000 realizations of the linear predictor for each 
# of the scenarios "correlated", "independent", and "highdim" and
# calculate empirical distribution function as an approximation of
# the true distribution function (needed for scenario "equal class 
# widths", see paper for details):

ynumbiguncor <- simynum(linpred=simlinpred(X=simpreds(n = 50000, correlations=FALSE, corstrength=0.8, ncor=6, nnoise=50, highdim=FALSE), 
                                      nnoise=50, effect11=0.5, effect12=0.75, effect13=1), sigmanoise=1)

qufununcor <- function(qu) 
  quantile(x=ynumbiguncor, probs=qu)


ynumbigcor <- simynum(linpred=simlinpred(X=simpreds(n = 50000, correlations=TRUE, corstrength=0.8, ncor=6, nnoise=50, highdim=FALSE), 
                                           nnoise=50, effect11=0.5, effect12=0.75, effect13=1), sigmanoise=1)

qufuncor <- function(qu) 
  quantile(x=ynumbigcor, probs=qu)


ynumbighighdim <- simynum(linpred=simlinpred(X=simpreds(n = 50000, correlations=TRUE, corstrength=0.8, ncor=6, nnoise=50, highdim=TRUE), 
                                         nnoise=50, effect11=0.5, effect12=0.75, effect13=1), sigmanoise=1)

qufunhighdim <- function(qu) 
  quantile(x=ynumbighighdim, probs=qu)




# Export all objects to the CPUs:

sfExportAll()



# Evaluate the settings:

Results <- sfLapply(1:nrow(scenariogrid), function(z) try({evaluatesetting(z)}))



# Stop the cluster:

sfStop()



# Save the results:

save(Results, file="./HornungOrdinalForests/Results/Simulation_Results.Rda")



# Clear the workspace:

rm(list=ls());gc()
