# In case the folder "HornungOrdinalForests" is not contained in the
# current R working directory, use the following line and replace
# *directory that contains the folder 'HornungOrdinalForests'* by
# the path to the directory that contains the folder
# "HornungOrdinalForests":

# setwd("*directory that contains the folder 'HornungOrdinalForests'*")


# Load and process results:
############################

# The R script "SimulationPerffunction.R" (subfolder "Simulation") produces an
# Rda file labeled "SimulationPerffunction_Results.Rda". This file
# is really large (32 MB), because it contains the
# true and predicted class values for each combination of test dataset
# and method.
# Therefore, this Rda file is not included in the electronic
# appendix (it can, however, be obtained using "SimulationPerffunction.R"),
# but instead a much smaller Rda file "SimulationPerffunction_Results_Metrics.Rda"
# is included that contains the calculated metrics instead
# of the true and predicted class values.
# This Rda file "SimulationPerffunction_Results_Metrics.Rda" is producable using
# the following code within "if(FALSE) {....}" (for this code, of course, 
# "SimulationPerffunction_Results.Rda" is required):

if(FALSE) {
  
  # Load simulation results:
  
  load("./HornungOrdinalForests/Results/SimulationPerffunction_Results.Rda")
  
  
  # Function to calculate micro-average:
  
  tprmicro <- function(ytrue, yhat) {
    mean(ytrue==yhat)
  }
  
  # Function to calculate macro-average:
  
  tprmacro <- function(ytrue, yhat) {
    classes <- unique(ytrue)
    mean(sapply(classes, function(x) sum(ytrue==x & yhat==x)/sum(ytrue==x)))
  }
  
  # Function to calculate sensitivity for the prediction of class 2:
  
  tprcl2 <- function(ytrue, yhat) {
    sum(ytrue==2 & yhat==2)/sum(ytrue==2)
  }
  
  
  # Calculate micro-average, macro-average, and sensitivity for the prediction
  # of class 2:
  
  tprmicroval <- sapply(Results, function(x) tprmicro(x$ytrue, x$yhat))
  tprmacroval <- sapply(Results, function(x) tprmacro(x$ytrue,x$yhat))
  tprcl2val <- sapply(Results, function(x) tprcl2(x$ytrue,x$yhat))
  
  Results <- mapply(function(x, y) c(x, 'tprmicro'=y), Results, tprmicroval, SIMPLIFY=FALSE)
  Results <- mapply(function(x, y) c(x, 'tprmacro'=y), Results, tprmacroval, SIMPLIFY=FALSE)
  Results <- mapply(function(x, y) c(x, 'tprcl2'=y), Results, tprcl2val, SIMPLIFY=FALSE)
  Results <- lapply(Results, function(x) {
    x$ytrue <- NULL
    x$yhat <- NULL
    return(x)
  })
  
  # Save processed results:
  
  save(Results, file="./HornungOrdinalForests/Results/SimulationPerffunction_Results_Metrics.Rda")
  
}

# Load the processed results:

load("./HornungOrdinalForests/Results/SimulationPerffunction_Results_Metrics.Rda")









# Make scenario table to match the results to the corresponing meta information:


usecorrelations_highdim_numberobs <- c("correlations_nonhighdim_200", "nocorrelations_nonhighdim_200", 
                                       "correlations_nonhighdim_400", "nocorrelations_nonhighdim_400", 
                                       "correlations_highdim")
equalclasswidths <- c(TRUE, FALSE)
numberclass <- c(3,6,9)


performancefunction <- c("equal", "proportional", "oneclass")

simit <- 1:50


scenariogrid <- expand.grid(performancefunction=performancefunction, simit=simit, equalclasswidths=equalclasswidths,  
                            numberclass=numberclass, usecorrelations_highdim_numberobs=usecorrelations_highdim_numberobs, stringsAsFactors = FALSE)
scenariogrid <- scenariogrid[,ncol(scenariogrid):1]



set.seed(1234)
seeds <- sample(1000:10000000, size=length(simit)*length(equalclasswidths)*length(numberclass)*length(usecorrelations_highdim_numberobs))

scenariogrid$seed <- rep(seeds, each=length(performancefunction))



set.seed(1234)
reorderind <- sample(1:nrow(scenariogrid))
scenariogrid <- scenariogrid[reorderind,]




# Make results table:

results <- scenariogrid

tempvar <- results$usecorrelations_highdim_numberobs

results$correlation <- TRUE
results$correlation[grep("nocorrelations", tempvar)] <- FALSE

results$highdim <- TRUE
results$highdim[grep("nonhighdim", tempvar)] <- FALSE

results$n <- 200
results$n[grep("400", tempvar)] <- 400

results$tprmicro <- sapply(Results, function(x) x$tprmicro)
results$tprmacro <- sapply(Results, function(x) x$tprmacro)
results$tprcl2 <- sapply(Results, function(x) x$tprcl2)





# Assign shorter or more adequate labels to setting parameter levels:

names(results)[names(results)=="usecorrelations_highdim_numberobs"] <- "covariates_n"
results$covariates_n[results$covariates_n=="correlations_nonhighdim_200"] <- "correlated_n200"
results$covariates_n[results$covariates_n=="nocorrelations_nonhighdim_200"] <- "independent_n200"
results$covariates_n[results$covariates_n=="correlations_nonhighdim_400"] <- "correlated_n400"
results$covariates_n[results$covariates_n=="nocorrelations_nonhighdim_400"] <- "independent_n400"
results$covariates_n[results$covariates_n=="correlations_highdim"] <- "highdim"

results$covariates_n <- factor(results$covariates_n, levels=c("correlated_n200", "correlated_n400", "independent_n200", "independent_n400", "highdim"))

names(results)[names(results)=="equalclasswidths"] <- "intervals"
results$intervals[results$intervals=="TRUE"] <- "equal width"
results$intervals[results$intervals=="FALSE"] <- "random width"

names(results)[names(results)=="numberclass"] <- "nclass"
results$nclass[results$nclass=="3"] <- "nclass = 3"
results$nclass[results$nclass=="6"] <- "nclass = 6"
results$nclass[results$nclass=="9"] <- "nclass = 9"

results$performancefunction <- factor(results$performancefunction, 
                                      levels=c("equal", "proportional", "oneclass"))




# Boxplots of values of micro-average, macro-average, and sensitivity for the prediction
# of class 2 for all settings:
########################################################################################

library("ggplot2")

xlabs <- c(expression(g["clequal"]),
           expression(g["clprop"]),
           expression(g["clj"]))

p <- ggplot(data=results[results$intervals=="equal width",], aes(x=performancefunction, y=tprmacro)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width=0.7) + theme_bw() + labs(x="performance function", y="MaA") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=13, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16)) + 
  scale_x_discrete(labels=xlabs)
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationEqualWidthtprmacroPerffunction.pdf", width=9*1.2, height=10*1.2)


p <- ggplot(data=results[results$intervals=="random width",], aes(x=performancefunction, y=tprmacro)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width=0.7) + theme_bw() + labs(x="performance function", y="MaA") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=13, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16)) + 
  scale_x_discrete(labels=xlabs)
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationRandomWidthtprmacroPerffunction.pdf", width=9*1.2, height=10*1.2)



p <- ggplot(data=results[results$intervals=="equal width",], aes(x=performancefunction, y=tprmicro)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width=0.7) + theme_bw() + labs(x="performance function", y="MiA") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=13, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16)) + 
  scale_x_discrete(labels=xlabs)
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationEqualWidthtprmicroPerffunction.pdf", width=9*1.2, height=10*1.2)


p <- ggplot(data=results[results$intervals=="random width",], aes(x=performancefunction, y=tprmicro)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width=0.7) + theme_bw() + labs(x="performance function", y="MiA") + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=13, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16)) + 
  scale_x_discrete(labels=xlabs)
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationRandomWidthtprmicroPerffunction.pdf", width=9*1.2, height=10*1.2)




p <- ggplot(data=results[results$intervals=="equal width",], aes(x=performancefunction, y=tprcl2)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width=0.7) + theme_bw() + labs(x="performance function", y=expression(TPR[2])) + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=13, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16)) + 
  scale_x_discrete(labels=xlabs)
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationEqualWidthtprcl2Perffunction.pdf", width=9*1.2, height=10*1.2)


p <- ggplot(data=results[results$intervals=="random width",], aes(x=performancefunction, y=tprcl2)) + facet_wrap(~ covariates_n + nclass, ncol=3, scales="free_y") + 
  geom_boxplot(width=0.7) + theme_bw() + labs(x="performance function", y=expression(TPR[2])) + 
  theme(axis.text.x=element_text(size=16, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=13, colour="black"),
        axis.title.x=element_blank(),
        axis.title.y=element_text(size=16),
        strip.text.x = element_text(size = 16)) + 
  scale_x_discrete(labels=xlabs)
p

ggsave("./HornungOrdinalForests/Results/Figures/FigureSimulationRandomWidthtprcl2Perffunction.pdf", width=9*1.2, height=10*1.2)





# Calculation that demonstrates that in the settings with 'nclass = 9' there
# are 7 and 14 observations in class 2 in the mean for n = 200 and n = 400, respectively:
#########################################################################################

# number of classes:
J <- 9

# Interval borders in the range of the latent continuous target variable
# in the scenario with equal class widths:
intervals <- seq(qnorm(0.001), qnorm(0.999), length=J+1)

# Probability that an observation is in class 2 in that scenario:
probclass2 <- pnorm(intervals[3]) - pnorm(intervals[2])

# Mean frequency of observations in class 2 in datasets of sizes 200 and 400, respectively:
(class2n200 <- round(probclass2*200))
(class2n400 <- round(probclass2*400))
