# Set working directory:

setwd("~/")



# Make table of settings:

nsetsvec <- c(500, 1000, 5000, 10000)
nbestvec <- c(5, 10, 100, 1000)

nsetsnbest <- expand.grid(nbestvec=nbestvec, nsetsvec=nsetsvec, stringsAsFactors = FALSE)
nsetsnbest <- nsetsnbest[,ncol(nsetsnbest):1]

nsetsnbest <- nsetsnbest[!(nsetsnbest$nsetsvec==500 & nsetsnbest$nbestvec==1000) & !(nsetsnbest$nsetsvec==1000 & nsetsnbest$nbestvec==1000),]
rownames(nsetsnbest) <- NULL

nsetsnbestsettings <- apply(nsetsnbest, 1, function(x) paste("nsets = ", x[1], ", nbest = ", x[2], sep=""))

ntreeperdivsettings <- paste("ntreeperdiv = ", c(10, 50, 100, 500, 1000), sep="")

ntreefinalsettings <- paste("ntreefinal = ", c(500, 1000, 5000, 10000), sep="")

npermtrialsettings <- paste("npermtrial = ", c(1, 100, 500, 1000))


paramsetting <- c(nsetsnbestsettings, ntreeperdivsettings, ntreefinalsettings, npermtrialsettings)


simsetting <- c("lowdim", "highdim")

datasetind <- 1:20


scenariogrid <- expand.grid(paramsetting=paramsetting, simsetting=simsetting, datasetind=datasetind, stringsAsFactors = FALSE)
scenariogrid <- scenariogrid[,ncol(scenariogrid):1]


set.seed(1234)
seeds <- sample(1000:10000000, size=length(simsetting)*length(datasetind))

scenariogrid$seed <- rep(seeds, each=length(paramsetting))




# Randomly permute rows of the table containing the settings.
# This is performed to ensure a comparable computational burdens for
# the jobs to be performed in parallel:

set.seed(1234)
reorderind <- sample(1:nrow(scenariogrid))
scenariogrid <- scenariogrid[reorderind,]




# Source the functions that are used in performing the calculations 
# on the cluster:

source("./HornungOrdinalForests/Functions/FunctionsSimulation.R")




# Generate 50,000 realizations of the linear predictor for each 
# of the scenarios "correlated" and "highdim" and
# calculate empirical distribution function as an approximation of
# the true distribution function (needed for scenario "equal class 
# widths", see paper for details):

ynumbigcor <- simynum(linpred=simlinpred(X=simpreds(n = 50000, correlations=TRUE, corstrength=0.8, ncor=6, nnoise=50, highdim=FALSE), 
                                         nnoise=50, effect11=0.5, effect12=0.75, effect13=1), sigmanoise=1)

qufuncor <- function(qu) 
  quantile(x=ynumbigcor, probs=qu)


ynumbighighdim <- simynum(linpred=simlinpred(X=simpreds(n = 50000, correlations=TRUE, corstrength=0.8, ncor=6, nnoise=50, highdim=TRUE), 
                                             nnoise=50, effect11=0.5, effect12=0.75, effect13=1), sigmanoise=1)

qufunhighdim <- function(qu) 
  quantile(x=ynumbighighdim, probs=qu)


 

# Export all objects to the CPUs:

sfExportAll()



# Evaluate the settings:

Results <- sfLapply(1:nrow(scenariogrid), function(z) try({evaluatesettinghyperparam(z)}))


# Stop the cluster:

sfStop()


# Save the results:

save(Results, file="./HornungOrdinalForests/Results/SimulationSensitivityAnalysisHyperparameters_Results.Rda")


# Clear the workspace:

rm(list=ls());gc()
