# In case the folder "HornungOrdinalForests" is not contained in the
# current R working directory, use the following line and replace
# *directory that contains the folder 'HornungOrdinalForests'* by
# the path to the directory that contains the folder
# "HornungOrdinalForests":

# setwd("*directory that contains the folder 'HornungOrdinalForests'*")


# Load and process results:
############################

# The R script "SimulationSensitivityAnalysisHyperparameters.R" (subfolder "Simulation") produces an
# Rda file labeled "SimulationSensitivityAnalysisHyperparameters_Results.Rda". This file
# is quite large (9 MB), because it contains the
# true and predicted class values for each evaluation on each test dataset.
# Therefore, this Rda file is not included in the electronic
# appendix (it can, however, be obtained using "SimulationSensitivityAnalysisHyperparameters.R"),
# but instead a much smaller Rda file "SimulationSensitivityAnalysisHyperparameters_Results_WithKappa.Rda"
# is included that contains the calculated linearly weighted Kappa values instead
# of the true and predicted class values.
# This Rda file "SimulationSensitivityAnalysisHyperparameters_Results_WithKappa.Rda" is producable using
# the following code within "if(FALSE) {....}" (for this code, of course, 
# "SimulationSensitivityAnalysisHyperparameters_Results.Rda" is required):  

if(FALSE) {
  
  # Load simulation results:
  
  load("./HornungOrdinalForests/Results/SimulationSensitivityAnalysisHyperparameters_Results.Rda")
  
  
  # Calculate linearly weighted Kappa:
  
  linearkappa <- function(ytrue, yhat) {
    
    require("psych")
    
    x <- data.frame(ytrue=ytrue, yhat=yhat)
    
    J <- length(unique(c(x$ytrue, x$yhat)))
    
    myw <- matrix(0, ncol = J, nrow = J)
    myw[] <- abs((col(myw) - row(myw)))
    myw <- 1 - myw/(J - 1)
    
    cohen.kappa(x, w=myw)$weighted.kappa
    
  }
  
  kappalin <- sapply(Results, function(x) linearkappa(x$ytrue, x$yhat))
  
  
  Results <- mapply(function(x, y) c(x, 'kappalin'=y), Results, kappalin, SIMPLIFY=FALSE)
  Results <- lapply(Results, function(x) {
    x$ytrue <- NULL
    x$yhat <- NULL
    return(x)
  })
  
  # Save processed results:
  
  save(Results, file="./HornungOrdinalForests/Results/SimulationSensitivityAnalysisHyperparameters_Results_WithKappa.Rda")
  
}

# Load the processed results:

load("./HornungOrdinalForests/Results/SimulationSensitivityAnalysisHyperparameters_Results_WithKappa.Rda")





# Make scenario table to match the results to the corresponing meta information:


nsetsvec <- c(500, 1000, 5000, 10000)
nbestvec <- c(5, 10, 100, 1000)

nsetsnbest <- expand.grid(nbestvec=nbestvec, nsetsvec=nsetsvec, stringsAsFactors = FALSE)
nsetsnbest <- nsetsnbest[,ncol(nsetsnbest):1]

nsetsnbest <- nsetsnbest[!(nsetsnbest$nsetsvec==500 & nsetsnbest$nbestvec==1000) & !(nsetsnbest$nsetsvec==1000 & nsetsnbest$nbestvec==1000),]
rownames(nsetsnbest) <- NULL

nsetsnbestsettings <- apply(nsetsnbest, 1, function(x) paste("nsets = ", x[1], ", nbest = ", x[2], sep=""))

ntreeperdivsettings <- paste("ntreeperdiv = ", c(10, 50, 100, 500, 1000), sep="")

ntreefinalsettings <- paste("ntreefinal = ", c(500, 1000, 5000, 10000), sep="")

npermtrialsettings <- paste("npermtrial = ", c(1, 100, 500, 1000))


paramsetting <- c(nsetsnbestsettings, ntreeperdivsettings, ntreefinalsettings, npermtrialsettings)


simsetting <- c("lowdim", "highdim")

datasetind <- 1:20



scenariogrid <- expand.grid(paramsetting=paramsetting, simsetting=simsetting, datasetind=datasetind, stringsAsFactors = FALSE)
scenariogrid <- scenariogrid[,ncol(scenariogrid):1]



set.seed(1234)
seeds <- sample(1000:10000000, size=length(simsetting)*length(datasetind))

scenariogrid$seed <- rep(seeds, each=length(paramsetting))


set.seed(1234)
reorderind <- sample(1:nrow(scenariogrid))
scenariogrid <- scenariogrid[reorderind,]




# Make results table:

results <- scenariogrid

results$kappalin <- sapply(Results, function(x) x$kappalin)



# Order rows in the table of results:

orderind <- order(results$simsetting, results$datasetind, results$paramsetting)
results <- results[orderind,]
Results <- Results[orderind]





# Boxplots of Kappa values for combination of B_sets and B_bestsets:
#####################################################################

resultstemp <- results[grep("nsets = ", results$paramsetting),]
dim(resultstemp)
unique(resultstemp$paramsetting)

resultstemp$paramsetting <- factor(resultstemp$paramsetting, levels=nsetsnbestsettings)


resultstemp$nsets <- sapply(as.character(resultstemp$paramsetting), function(x) strsplit(x, split=",")[[1]][1])
resultstemp$nbest <- sapply(as.character(resultstemp$paramsetting), function(x) strsplit(x, split=", ")[[1]][2])

resultstemp$nsets <- factor(resultstemp$nsets, levels=c("nsets = 500", "nsets = 1000", "nsets = 5000", "nsets = 10000"))
resultstemp$nbest <- factor(resultstemp$nbest, levels=c("nbest = 5", "nbest = 10", "nbest = 100", "nbest = 1000"))

resultstemp <- resultstemp[order(resultstemp$nsets, resultstemp$nbest),]


labsx <- ""
for(i in 1:nrow(nsetsnbest))
  eval(parse(text=paste("labsx[i] <- expression(paste(B[\"sets\"], \" = \", ", nsetsnbest[i,1], ", \", \", B[\"bestsets\"], \" = \", ", nsetsnbest[i,2], ", sep=\"\"))", sep="")))

 
library("ggplot2")
p1 <- ggplot(data=resultstemp[resultstemp$simsetting=="lowdim",], aes(x=paramsetting, y=kappalin)) + 
  geom_boxplot() + ylab("Linearly weighted Kappa") + ggtitle("Low-dimensional data") + geom_vline(xintercept=c(3.5, 6.5, 10.5)) +theme_bw() + 
  theme(axis.text.x=element_text(size=14, angle=56.5, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        axis.title.y=element_text(size=16), plot.title=element_text(size=17)) + scale_x_discrete(labels=labsx)

p2 <- ggplot(data=resultstemp[resultstemp$simsetting=="highdim",], aes(x=paramsetting, y=kappalin)) + 
  geom_boxplot() + ylab("Linearly weighted Kappa") + ggtitle("High-dimensional data") + geom_vline(xintercept=c(3.5, 6.5, 10.5)) +theme_bw() + 
  theme(axis.text.x=element_text(size=14, angle=56.5, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        axis.title.y=element_text(size=16), plot.title=element_text(size=17)) + scale_x_discrete(labels=labsx)

library("gridExtra")
library("grid")
pall <- grid.arrange(p1, p2, nrow = 2)

ggsave(file="./HornungOrdinalForests/Results/Figures/FigureHyperparametersI.pdf", plot=pall, width=10*1.1, height=10*1.1)







# Boxplots of Kappa values for B_ntreeprior, B_ntree, and N_perm:
#################################################################

# B_ntreeprior:

resultstemp <- results[grep("ntreeperdiv = ", results$paramsetting),]
dim(resultstemp)
unique(resultstemp$paramsetting)

resultstemp$paramsetting <- factor(resultstemp$paramsetting, levels=ntreeperdivsettings)

xlabs1 <- gsub("ntreeperdiv = ", "", ntreeperdivsettings)

xlabs <- ""
for(i in seq(along=xlabs1))                          
eval(parse(text=paste("xlabs[i] <- expression(paste(B[\"ntreeprior\"], \" = \", ", xlabs1[i], ", sep=\"\"))", sep="")))
                          
p3 <- ggplot(data=resultstemp[resultstemp$simsetting=="lowdim",], aes(x=paramsetting, y=kappalin)) + 
  geom_boxplot() + ylab("Linearly weighted Kappa") + ggtitle("Low-dimensional data") + theme_bw() + 
  theme(axis.text.x=element_text(size=14, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        axis.title.y=element_text(size=15), plot.title=element_text(size=17)) +
  scale_x_discrete(labels=xlabs)

p4 <- ggplot(data=resultstemp[resultstemp$simsetting=="highdim",], aes(x=paramsetting, y=kappalin)) + 
  geom_boxplot() + ylab("Linearly weighted Kappa") + ggtitle("High-dimensional data") + theme_bw() + 
  theme(axis.text.x=element_text(size=14, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        axis.title.y=element_text(size=15), plot.title=element_text(size=17)) +
  scale_x_discrete(labels=xlabs)


# B_ntree:

resultstemp <- results[grep("ntreefinal = ", results$paramsetting),]
dim(resultstemp)
unique(resultstemp$paramsetting)

resultstemp$paramsetting <- factor(resultstemp$paramsetting, levels=ntreefinalsettings)


xlabs1 <- gsub("ntreefinal = ", "", ntreefinalsettings)

xlabs <- ""
for(i in seq(along=xlabs1))                          
  eval(parse(text=paste("xlabs[i] <- expression(paste(B[\"ntree\"], \" = \", ", xlabs1[i], ", sep=\"\"))", sep="")))


p5 <- ggplot(data=resultstemp[resultstemp$simsetting=="lowdim",], aes(x=paramsetting, y=kappalin)) + 
  geom_boxplot() + ylab("Linearly weighted Kappa") + ggtitle("Low-dimensional data") + theme_bw() +
  theme(axis.text.x=element_text(size=14, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        axis.title.y=element_text(size=15), plot.title=element_text(size=17)) +
  scale_x_discrete(labels=xlabs)

p6 <- ggplot(data=resultstemp[resultstemp$simsetting=="highdim",], aes(x=paramsetting, y=kappalin)) + 
  geom_boxplot() + ylab("Linearly weighted Kappa") + ggtitle("High-dimensional data") + theme_bw() +
  theme(axis.text.x=element_text(size=14, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        axis.title.y=element_text(size=15), plot.title=element_text(size=17)) +
  scale_x_discrete(labels=xlabs)


# N_perm:

resultstemp <- results[grep("npermtrial = ", results$paramsetting),]
dim(resultstemp)
unique(resultstemp$paramsetting)

resultstemp$paramsetting <- factor(resultstemp$paramsetting, levels=npermtrialsettings)


xlabs1 <- gsub("npermtrial = ", "", npermtrialsettings)

xlabs <- ""
for(i in seq(along=xlabs1))                          
  eval(parse(text=paste("xlabs[i] <- expression(paste(N[\"perm\"], \" = \", ", xlabs1[i], ", sep=\"\"))", sep="")))

p7 <- ggplot(data=resultstemp[resultstemp$simsetting=="lowdim",], aes(x=paramsetting, y=kappalin)) + 
  geom_boxplot() + ylab("Linearly weighted Kappa") + ggtitle("Low-dimensional data") + theme_bw() +
  theme(axis.text.x=element_text(size=14, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        axis.title.y=element_text(size=15), plot.title=element_text(size=17)) +
  scale_x_discrete(labels=xlabs)

p8 <- ggplot(data=resultstemp[resultstemp$simsetting=="highdim",], aes(x=paramsetting, y=kappalin)) + 
  geom_boxplot() + ylab("Linearly weighted Kappa") + ggtitle("High-dimensional data") + theme_bw() +
  theme(axis.text.x=element_text(size=14, angle=45, vjust=1, hjust=1, colour="black"), 
        axis.text.y=element_text(size=14, colour="black"), axis.title.x=element_blank(),
        axis.title.y=element_text(size=15), plot.title=element_text(size=17)) +
  scale_x_discrete(labels=xlabs)



# Combine the plots:

p34 <- arrangeGrob(p3, p4, ncol=2, nrow=1)
p56 <- arrangeGrob(p5, p6, ncol=2, nrow=1)
p78 <- arrangeGrob(p7, p8, ncol=2, nrow=1)

pall <- grid.arrange(p34, p56, p78, nrow = 3)#, heights=c(1,6))


ggsave(file="./HornungOrdinalForests/Results/Figures/FigureHyperparametersII.pdf", plot=pall, width=8*1.3, height=9*1.3)
